---------------------------------------------------------------------------------------------------------------------------------
--	Sierra Automated Systems
--	Clock disturbance monitor state machine
--
--	Author:	Anthony Medina
--	Date:	Apr 7, 2017
--
--	Description: This logic will monitor a 'Dead' clock state or 'Phase' align error in BP LRCLK and Dante LRCLK falling edge
--				 The LRCLK edges will be within a +/- 1uS window of each other when in lock. Any disturbanace should be caught
--				 and result in resetting of both the Frame DCM and Dante side DCM to 'ReSync' everyone
--
--				 In the case of a 'Dead' clock where the Frame looses clock for a long period then the same thing happens,
--				 both Frame and Dante side DCM's get reset.
--
--				 The goal is to manage the time when either condition occurs so that enough time is allowed for the system to
--				 stabilize before monitoring for the fault conditions.
--
--				 Without this then a cycle of phase align error then dead clock continues and the system continually resets
--
--
---------------------------------------------------------------------------------------------------------------------------------
LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

Entity ClockDisturbance_SM is
	GENERIC(WIDTH	: INTEGER	:= 24);
Port(
		POR			:IN 	STD_LOGIC;	--	Power On Reset
		DeadClk		:IN 	STD_LOGIC;	--	Check for dead clock
		ClkPhse		:IN 	STD_LOGIC;	--	Check for BP_LRCLK / D_LRCLK phase align, +/- 1uS Falling edge / Falling edge
		Clk			:IN		STD_LOGIC;	--	Clock for SM
		LRClk		:IN		STD_LOGIC;	--	Clock for counters
		Reset		:IN 	STD_LOGIC;	--	Generic reset
		StateOut	:OUT	STD_LOGIC_VECTOR(3 DOWNTO 0);	--	Monitor state, easier to tell what activated
		DeadClk_RST	:OUT	STD_LOGIC;	--	Terminal counter1 signal, high is counting
		ClkPhse_RST	:OUT	STD_LOGIC	--	Terminal counter2 signal, hihg is counting
	);
End ClockDisturbance_SM;

Architecture Behave of ClockDisturbance_SM is
--	State signals
	Type	State_Type is (S0,S1,S2,S3,S4);			--	type of state machine.
	SIGNAL	Curr_State, Next_State	:	State_Type;	--	current and next state declaration.

--	Counter signals
	SIGNAL	Cnt1_Rst, Cnt2_Rst, Cnt3_Rst	:	STD_LOGIC	:=	'0';
	SIGNAL	QTMP1, QTMP2, QTMP3				:	STD_LOGIC_VECTOR (WIDTH-1 DOWNTO 0)	:=	(OTHERS => '0');
	SIGNAL	TCNT1, TCNT2, TCNT3				:	STD_LOGIC	:=	'0';
	SIGNAL	Cnt1_Ena, Cnt2_Ena, Cnt3_Ena	:	STD_LOGIC	:=	'0';
	
Begin

	ClkPhse_RST	<=	TCNT1;
	DeadClk_RST	<=	TCNT2;

Process (Clk,Reset,POR)
Begin
	if(Reset = '1' OR POR = '1') then
		Curr_State	<=	S0;			-- Defalut on reset.
	elsif(Rising_Edge(Clk)) then
		Curr_State	<=	Next_State;	-- state change.
	end if;
End Process;

--state machine process.
Process (Curr_State,POR,DeadClk,ClkPhse)
Begin
	Case Curr_State is
---------------------------------------------------------------------------------------------------------------------------------
		When S0 =>			-- S0 is POR state
		Cnt1_Ena	<=	'0';
		Cnt2_Ena	<=	'0';
		Cnt3_Ena	<=	'0';
		Cnt1_Rst	<=	'0';
		Cnt2_Rst	<=	'0';
		Cnt3_Rst	<=	'0';
		if(POR = '0') then
			StateOut	<=	CONV_STD_LOGIC_VECTOR(1,4);
			Next_State	<=	S1;
		else
			StateOut	<=	CONV_STD_LOGIC_VECTOR(0,4);
			Next_State	<=	S0;
		end if;   
---------------------------------------------------------------------------------------------------------------------------------
		When S1 =>			--	S1 is ClockDisturbance Monitor state
		if(DeadClk = '1') then
			StateOut	<= CONV_STD_LOGIC_VECTOR(2,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'1';
			Cnt2_Rst	<=	'0';
			Cnt3_Rst	<=	'1';
			Next_State	<=	S2;
		elsif(ClkPhse = '1') then
			StateOut	<= CONV_STD_LOGIC_VECTOR(3,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'0';
			Cnt2_Rst	<=	'1';
			Cnt3_Rst	<=	'1';
			Next_State	<=	S3;
		else
			StateOut	<= CONV_STD_LOGIC_VECTOR(1,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'0';
			Cnt2_Rst	<=	'0';
			Cnt3_Rst	<=	'0';
			Next_State	<=	S1;
		end if;
---------------------------------------------------------------------------------------------------------------------------------
		When S2 =>			--	S2 is DeadClk state
--		Cnt1_Rst	<=	'0';
--		Cnt2_Rst	<=	'0';
		Cnt3_Rst	<=	'0';
		if(TCNT1 = '0') then
			StateOut	<= CONV_STD_LOGIC_VECTOR(0,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'0';
			Cnt2_Rst	<=	'1';
			Next_State	<=	S3;
		elsif(DeadClk = '1') then	--	This keeps the counter in reset which keeps all internal FPGA logic/DCM in reset
			Cnt1_Ena	<=	'0';	--	Until the dead clock state is exited (clocks come back in frame)
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'1';	--	Keeps the DeadClk state count in reset which keeps TCNT1 'HI'
			Cnt2_Rst	<=	'0';
			Next_State	<=	S2;
		else
			StateOut	<= CONV_STD_LOGIC_VECTOR(2,4);
			Cnt1_Ena	<=	'1';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Cnt1_Rst	<=	'0';
			Cnt2_Rst	<=	'0';
			Next_State	<=	S2;
		end if;
---------------------------------------------------------------------------------------------------------------------------------
		When S3 =>			--	S3 is ClkPhse state
		Cnt1_Rst	<=	'0';
		Cnt2_Rst	<=	'0';
		Cnt3_Rst	<=	'0';
		if(TCNT2 = '0') then
			StateOut	<= CONV_STD_LOGIC_VECTOR(0,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Next_State	<=	S4;
		else
			StateOut	<= CONV_STD_LOGIC_VECTOR(3,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'1';
			Cnt3_Ena	<=	'0';
			Next_State	<=	S3;
		end if;
---------------------------------------------------------------------------------------------------------------------------------
		When S4 =>			--	S4 is Final stabilizing state to allow system to settle
		Cnt1_Rst	<=	'0';
		Cnt2_Rst	<=	'0';
		Cnt3_Rst	<=	'0';
		if(TCNT3 = '0') then
			StateOut	<= CONV_STD_LOGIC_VECTOR(0,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'0';
			Next_State	<=	S0;
		else
			StateOut	<= CONV_STD_LOGIC_VECTOR(4,4);
			Cnt1_Ena	<=	'0';
			Cnt2_Ena	<=	'0';
			Cnt3_Ena	<=	'1';
			Next_State	<=	S4;
		end if;
	End Case;
End Process;
---------------------------------------------------------------------------------------------------------------------------------
--	Use 3 for simulation.
--	1,440,000	Dec	=>	15F900	Hex:	30	seconds,	16#15F900#
--	960,000		Dec	=>	0EA600	Hex:	20	seconds,	16#0EA600#
--	480,000		Dec	=>	075300	Hex:	10	seconds,	16#075300#
--	144,000		Dec	=>	23280	Hex:	3	seconds,	16#23280#
--	 48,000		Dec	=>	BB80	Hex:	1	second,		16#BB80#
--
--	Try to keep this as minimal as possible but enough to allow the Dante
--	to boot up and switch over SCLK from base 3MHz to 24MHz.
--	This takes approximately 3 to 7 seconds
--
--	CONSTANT	TIMER			:	INTEGER	:=	16#15F900#;	--	30	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#0EA600#;	--	20	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#075300#;	--	10	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#03A980#;	--	5	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#02EE00#;	--	4	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#023280#;	--	3	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#00BB80#;	--	1	second
--	CONSTANT	TIMER			:	INTEGER	:=	16#008340#;	--	700 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#005A00#;	--	480 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#000003#;	--	Use this for simulation only
---------------------------------------------------------------------------------------------------------------------------------
COUNTER1:PROCESS(LRClk,POR,Cnt1_Rst,Cnt1_Ena,QTMP1)
--	CONSTANT	TIMER			:	INTEGER	:=	16#000100#;	--	Simulation only
--	CONSTANT	TIMER			:	INTEGER	:=	16#02EE00#;	--	4	seconds
	CONSTANT	TIMER			:	INTEGER	:=	16#03A980#;	--	5	seconds
BEGIN
	IF (POR = '1') THEN
		QTMP1	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT1	<=	'0';
	ELSIF(Cnt1_Rst = '1') THEN
		QTMP1	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT1	<=	'1';
	ELSIF RISING_EDGE(LRClk) THEN
		IF(Cnt1_Ena = '1') THEN
			IF(QTMP1 > CONV_STD_LOGIC_VECTOR(TIMER,WIDTH)) THEN
				QTMP1	<=	QTMP1;
				TCNT1	<=	'0';
			ELSE
				QTMP1	<= QTMP1 + '1';
				TCNT1	<=	'1';
			END IF;
		END IF;
	ELSE		
	END IF;
END PROCESS COUNTER1;
---------------------------------------------------------------------------------------------------------------------------------
COUNTER2:PROCESS(LRClk,POR,Cnt2_Rst,Cnt2_Ena,QTMP2)
--	CONSTANT	TIMER2			:	INTEGER	:=	16#00100#;	--	Simulation only
--	CONSTANT	TIMER2			:	INTEGER	:=	16#2EE00#;	--	4	seconds
	CONSTANT	TIMER2			:	INTEGER	:=	16#03A980#;	--	5	seconds
BEGIN
	IF (POR = '1') THEN
		QTMP2	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT2	<=	'0';
	ELSIF(Cnt2_Rst = '1') THEN
		QTMP2	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT2	<=	'1';
	ELSIF RISING_EDGE(LRClk) THEN
		IF(Cnt2_Ena = '1') THEN
			IF(QTMP2 > CONV_STD_LOGIC_VECTOR(TIMER2,WIDTH)) THEN
				QTMP2	<=	QTMP2;
				TCNT2	<=	'0';
			ELSE
				QTMP2	<= QTMP2 + '1';
				TCNT2	<=	'1';
			END IF;
		END IF;
	ELSE		
	END IF;
END PROCESS COUNTER2;
---------------------------------------------------------------------------------------------------------------------------------
COUNTER3:PROCESS(LRClk,POR,Cnt3_Rst,Cnt3_Ena,QTMP3)
--	CONSTANT	TIMER3			:	INTEGER	:=	16#00100#;	--	Simulation only
--	CONSTANT	TIMER3			:	INTEGER	:=	16#075300#;	--	10	seconds
	CONSTANT	TIMER3			:	INTEGER	:=	16#03A980#;	--	5	seconds
BEGIN
	IF (POR = '1') THEN
		QTMP3	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT3	<=	'0';
	ELSIF(Cnt3_Rst = '1') THEN
		QTMP3	<= CONV_STD_LOGIC_VECTOR(0,WIDTH);
		TCNT3	<=	'1';
	ELSIF RISING_EDGE(LRClk) THEN
		IF(Cnt3_Ena = '1') THEN
			IF(QTMP3 > CONV_STD_LOGIC_VECTOR(TIMER3,WIDTH)) THEN
				QTMP3	<=	QTMP3;
				TCNT3	<=	'0';
			ELSE
				QTMP3	<= QTMP3 + '1';
				TCNT3	<=	'1';
			END IF;
		END IF;
	ELSE		
	END IF;
END PROCESS COUNTER3;

End Behave;
