--===================================================================================================
-- Company: 		SIERRA AUTOMATED SYSTEMS
-- Engineer:			ANTHONY MEDINA
-- 
-- Create Date:		09:10:24 01/11/2010 
-- Design Name:	DANTE RX FIFO READ LOGIC
-- Module Name:	DANTE_RX_FIFO_READ - BEHAVE 
-- Project Name:	SUPER RIO
-- Target Devices:	SPARTAN-3ADSP
-- Tool versions:	ISE
-- Description:		This function waits until there is at least the minimum frames before beginning transfer to DMSTOR.
--							The frame size is 64 channels and the FIFO is designed to hold up to 16 frames (16 x 64 = 1024 chnls).
--							Upon every backplane RESYNC and Bankswap, 64 channels are read from the RX FIFO and written to
--							the DMSTOR dual bank memory in burst fashion.
--							Default frame count is set to 1. Due to the nature of the Dante and backplane resyncs, 
--							as long as the Dante and (32KD or SuperRIO) are 'rate' locked, there will be at least 2 frames of delay, 
--							one for the FIFO and one for the DMSTOR. At most there will be 3 frame delays if the frame count is set to one.
--							Depending on where the sample rate word clocks lock relative to each other.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--===================================================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY DANTE_RX_FIFO_READ is
	PORT(
				D_RXFIFO_CHNCNT		:IN	STD_LOGIC_VECTOR (12 DOWNTO 0);
				D_RXFIFO_RDVALID		:IN	STD_LOGIC;
				D_RSYNC_RESET			:IN	STD_LOGIC;
				BP_FRAME_DELAY			:IN	STD_LOGIC_VECTOR	(12 DOWNTO 0);
				BP_RESYNC					:IN	STD_LOGIC;
				BP_DMSTOR_BSWAP		:IN	STD_LOGIC;
				BP_CLK							:IN	STD_LOGIC;
				RST								:IN	STD_LOGIC;
				STATE							:OUT	STD_LOGIC_VECTOR	(2 DOWNTO 0);
				D_RXFIFO_RDEN			:OUT	STD_LOGIC;
				D_RXFIFO_XFREN			:OUT	STD_LOGIC;
				BP_DMSTOR_WADDR		:OUT	STD_LOGIC_VECTOR (4 DOWNTO 0);
				BP_DMSTOR_WREN		:OUT	STD_LOGIC
			);
END DANTE_RX_FIFO_READ;

ARCHITECTURE BEHAVE of DANTE_RX_FIFO_READ is

TYPE state_type is (S0, S1, S2, S3, S4, S5);
							-- S0 (000): WAIT FOR BACKPLANE RESYNC AND DANTE RX FIFO HAS AT LEAST MIN FRAME COUNT
							-- S1 (001): WAIT FOR BACKPLANE DMSTOR BANKSWAP, TOGGLE PULSE
							-- S2 (010): READ ONE CLOCK CYCLE FROM FIFO
							-- S3 (011): WAIT FOR A READ VALID PULSE
							-- S4 (100): BETWEEN S1 AND S4, WRITE 64 CHANNELS INTO DMSTOR THEN WAIT UNTIL NEXT DMSTOR BANKSWAP
							--				: AFTER 64 CHANNELS HAVE BEEN WRITTEN, GOTO S5
							-- S5 (101); CHECK TO SEE IF DANTE HAD TO RESET ITSELF, IF SO THEN RESET THE FIFO AND GOTO S0, ELSE S1

SIGNAL CURRENT_STATE, NEXT_STATE			:	state_type;										--	STATE transitions
SIGNAL STATE_XFER										:	STD_LOGIC_VECTOR (2 downto 0);	--	Reflect current state
SIGNAL ENA_XFER, XFER_ENA, TRANSFER		:	STD_LOGIC;										--	Internal WRITE transfer signals
SIGNAL	D_RXFIFO_READ, D_RDEN, D_READ	:	STD_LOGIC;										--	Internal READ transfer signals 
SIGNAL PULSECOUNT										:	STD_LOGIC_VECTOR (6 downto 0);	--	DMSTOR write address counter

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
REGISTERS: PROCESS(BP_CLK, RST, STATE_XFER, XFER_ENA, D_RDEN)			--
	VARIABLE	TCNT	: INTEGER	RANGE 32 DOWNTO 0	:=	0;						--	6-bit counter
BEGIN																											--
																													--
	IF RST = '1' THEN																						--
		BP_DMSTOR_WREN	<=		'0';															--	Backplane DMSTOR write ena
		BP_DMSTOR_WADDR	<=	"00000";														--	Backplane DMSTOR write address
		D_RXFIFO_READ		<=		'0';															--	iDante RX FIFO read
		D_READ					<=		'0';															--	iDante RX FIFO read ena
		CURRENT_STATE		<=		S0;															--	Async reset to RSYNC state
		TRANSFER					<=		'0';															--	Make sure no transfer occur
		ENA_XFER					<=		'0';															--
		STATE						<=	"000";															--	State output to zero
		TCNT						:=		0;																--	Set pulse count to zero	
	ELSIF RISING_EDGE(BP_CLK) THEN															--
		CURRENT_STATE	<=	NEXT_STATE;													--
		STATE					<=	STATE_XFER;													--	Output reflects state
		TRANSFER				<=	XFER_ENA;														--	Enables DMSTOR write transfer
		D_READ				<=	D_RDEN;															--	Enables DRXFIFO reading
			IF (TRANSFER	=	'1') THEN																--
				ENA_XFER	<=	'1';																		--	
					IF (D_READ	=	'1') THEN															--	This should enable READ for one clock cycle
						IF(D_RXFIFO_READ	=	'0')THEN											--
							D_RXFIFO_READ	<=	NOT D_RXFIFO_READ;					--	Toggle READ 
						ELSE																						--
							D_RXFIFO_READ	<=	'0';												--
						END IF;																					--
					ELSIF(D_RXFIFO_RDVALID	=	'1')THEN										--	During ENA_XFER HI
						IF(TCNT	<	32)THEN															--
							TCNT 	:=	TCNT + 1;														--	Inc the channel counter
							D_RXFIFO_READ	<=	'0';												--
						ELSE																						--
							TCNT	:=	TCNT;															--	Otherwise just hold the count
							D_RXFIFO_READ	<=	'0';												--
						END IF;																					--
					ELSE																							--
					END IF;																						--
			ELSE																									-- 
				TCNT					:=	0;																--
				ENA_XFER				<=	'0';															--	Keep FIFO write zero
				D_RXFIFO_READ	<=	'0';															--	Keep FIFO read zero
			END IF;																								--
		PULSECOUNT						<=	CONV_STD_LOGIC_VECTOR(TCNT,6);	--	Async combinatorial section uses this to shut off the transfers
--	Physical outputs
		D_RXFIFO_XFREN				<=	ENA_XFER;											--	RX transfer enable cycle
		D_RXFIFO_RDEN				<=	D_RXFIFO_READ;									--	Dante RX FIFO read enable
		BP_DMSTOR_WREN			<=	D_RXFIFO_RDVALID;								--	Backplane DMSTOR write enable
		BP_DMSTOR_WADDR(0)		<=	PULSECOUNT(0);									--	Backplane DMSTOR write address 0
		BP_DMSTOR_WADDR(1)		<=	PULSECOUNT(1);									--	Backplane DMSTOR write address 1
		BP_DMSTOR_WADDR(2)		<=	PULSECOUNT(2);									--	Backplane DMSTOR write address 2
		BP_DMSTOR_WADDR(3)		<=	PULSECOUNT(3);									--	Backplane DMSTOR write address 3
		BP_DMSTOR_WADDR(4)		<=	PULSECOUNT(4);									--	Backplane DMSTOR write address 4
	END IF;

END PROCESS REGISTERS;

--===================================================================================================
-- ASYNC PROCESS
--===================================================================================================
TRANSITIONS: PROCESS(	CURRENT_STATE, ENA_XFER, PULSECOUNT, D_RXFIFO_READ, BP_FRAME_DELAY, BP_RESYNC, BP_DMSTOR_BSWAP,
										D_RSYNC_RESET, D_RXFIFO_CHNCNT, D_RXFIFO_RDVALID)
BEGIN
	CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>							-- STATE 0, WAIT FOR BP RESYNC AND FRAME DELAY COUNT MET
		XFER_ENA	<=	'0';					--	ONCE STARTED THIS STATE IS NEVER REACHED UNLESS FOR SOME
		D_RDEN	<=	'0';					--	REASON THE DANTE I/F NEEDED TO RESET ITSELF
			IF ( BP_RESYNC	=	'1' AND (D_RXFIFO_CHNCNT	>=	BP_FRAME_DELAY) ) THEN
				NEXT_STATE	<=	S1;		-- WAIT FOR BACKPLANE BANKSWAP
				STATE_XFER	<=	"001";
			ELSE
				NEXT_STATE	<=	S0;		-- IF NOT THEN GO BACK TO WAIT FOR NEXT RESYNC
				STATE_XFER	<=	"000";
			END IF;
--===================================================================================================
		WHEN S1	=>							-- STATE 1, WAIT FOR BACKPLANE BANKSWAP TOGGLE
			IF ( BP_DMSTOR_BSWAP	=	'1' ) THEN
				NEXT_STATE	<=	S2;		-- WHEN BANKSWAP DETECTED START TRANSFER PROCESS
				STATE_XFER	<=	"010";
				XFER_ENA		<=	'1';		-- ENABLE TRANSFER
				D_RDEN		<=	'1';		-- ENABLE READ
			ELSE
				XFER_ENA		<=	'0';		-- DISABLE TRANSFER
				D_RDEN		<=	'0';		-- DISABLE READ
				NEXT_STATE	<=	S1;		-- ELSE WAIT UNTIL BANKSWAP
				STATE_XFER	<=	"001";
			END IF;
--===================================================================================================
		WHEN S2	=>							-- STATE 2, READ FIFO FOR ONE CLOCK CYCLE
			IF ( D_RXFIFO_READ	=	'1' ) THEN
				XFER_ENA		<=	'1';		-- ENABLE TRANSFER
				D_RDEN		<=	'0';
				NEXT_STATE	<=	S3;		-- WAIT FOR READ ACK
				STATE_XFER	<=	"011";
			ELSE
				XFER_ENA		<=	'1';		-- ENABLE TRANSFER
				D_RDEN		<=	'1';		-- ENABLE READ
				NEXT_STATE	<=	S2;		-- 
				STATE_XFER	<=	"010";
			END IF;
--===================================================================================================
		WHEN S3	=>							-- STATE 3, READ ACKNOWLEDGE
			IF ( D_RXFIFO_RDVALID	=	'1' ) THEN
				D_RDEN		<=	'0';
				XFER_ENA		<=	'1';
				NEXT_STATE	<=	S4;		-- IF READ CORRECTLY THEN WRITE IT
				STATE_XFER	<=	"100";
			ELSE
				D_RDEN		<=	'0';
				XFER_ENA		<=	'1';
				NEXT_STATE	<=	S3;		-- WAIT FOR READ ACK
				STATE_XFER	<=	"011";
			END IF;
--===================================================================================================
		WHEN S4	=>							-- STATE 4, WRITE DMSTOR
			IF ( PULSECOUNT	=	"100000" AND ENA_XFER =	'1' ) THEN
				XFER_ENA		<=	'0';
				D_RDEN		<=	'0';
				NEXT_STATE	<=	S5;		--	WHEN DONE WRITING 64 CHANNELS, CHECK IF DANTE RESET.
				STATE_XFER	<=	"101";	-- STATE 5 SELECTS RESYNC STATE 0 OR BANKSWAP TOGGLE STATE 1
			ELSE
				XFER_ENA		<=	'1';
				D_RDEN		<=	'1';
				NEXT_STATE	<=	S2;		--	ELSE KEEP READING/WRITING BY GOING BACK TO STATE 2
				STATE_XFER	<=	"010";
			END IF;
--===================================================================================================
		WHEN S5	=>							-- STATE 5, IF DANTE HAD TO RESET RESYNC, THIS MEANS A DISRUPTION IN DANTE I/F FOR SOME REASON.
		XFER_ENA	<=	'0';					--	IF SO THEN FIFO AND BACKPLANE DATA TRANSFER WILL BE OUT OF SYNC AND MOST LIKELY GARBAGE.
		D_RDEN	<=	'0';					--	BETTER TO RESET PROCESS, FIFO AND START OVER, SEND SILENCE AS IF FROM THE BEGINNING.
			IF ( D_RSYNC_RESET	=	'1' ) THEN
				NEXT_STATE	<=		S0;	-- WAIT FOR BACKPLANE RESYNC AND START OVER
				STATE_XFER	<=	"000";
			ELSE
				NEXT_STATE	<=		S1;	--	ELSE WAIT FOR NEXT BACKPLANE BANKSWAP
				STATE_XFER	<=	"001";
			END IF;
--===================================================================================================
--===================================================================================================
	END CASE;
END PROCESS TRANSITIONS;

END BEHAVE;
