--===================================================================================================
-- Company: 		SIERRA AUTOMATED SYSTEMS
-- Engineer:			ANTHONY MEDINA
-- 
-- Create Date:		10:22:24 01/05/2010 
-- Design Name:	DANTE RX FIFO WRITE LOGIC
-- Module Name:	DANTE_RX_FIFO_WRITE - BEHAVE 
-- Project Name:	SUPER RIO
-- Target Devices:	SPARTAN-3ADSP
-- Tool versions:	ISE
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--===================================================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY DANTE_RX_FIFO_WRITE is
	PORT(
				D_RESYNC	:IN	STD_LOGIC;
				D_CHNCNT	:IN	STD_LOGIC_VECTOR	(3 DOWNTO 0);
				D_CHNHLD	:IN	STD_LOGIC;
				D_SCLK		:IN	STD_LOGIC;
				RST				:IN	STD_LOGIC;
				STATE			:OUT	STD_LOGIC_VECTOR	(2 DOWNTO 0);
				D_CHNSEL	:OUT	STD_LOGIC_VECTOR	(1 DOWNTO 0);
				D_FIFO_WR	:OUT	STD_LOGIC
			);
END DANTE_RX_FIFO_WRITE;

ARCHITECTURE BEHAVE of DANTE_RX_FIFO_WRITE is

TYPE state_type is (S0, S1, S2, S3, S4, S5);
							-- S0 (000): DANTE RESYNC STATE, CHANNEL COUNT = 14 AND RESYNC PULSE
							-- S1 (001): DANTE CHANNEL COUNT GOES TO 15 FIRST TIME AFTER DANTE RESYNC
							-- S2 (010): WAIT FOR DANTE CHANNEL HOLD TO GO HI
							-- S3 (011): WAIT FOR DANTE CHANNEL HOLD TO GO LO, THIS IS NEEDED SINCE THE BP CLK IS FASTER
							-- S4 (100): WRITES 4 CHANNELS INTO FIFO ONCE DATA IS LOADED INTO HOLD REGISTERS FROM DANTE
							-- S5 (101): DANTE RESYNC CHECK, MAKE SURE DANTE IS STILL IN PROPER RESYNC STATE.
							-- IF NOT THEN GOTO RESYNC STATE AND START OVER

SIGNAL CURRENT_STATE, NEXT_STATE		:	state_type;
SIGNAL STATE_XFER									:	STD_LOGIC_VECTOR (2 downto 0);
SIGNAL ENA_XFER, XFER_ENA, TRANSFER	:	STD_LOGIC;
SIGNAL PULSECOUNT									:	STD_LOGIC_VECTOR (2 downto 0);

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
REGISTERS: PROCESS(D_SCLK, RST, STATE_XFER, XFER_ENA)
	VARIABLE	TCNT	: INTEGER	RANGE 7 DOWNTO 0	:=	0;		-- 3-bit counter
BEGIN
	IF RST = '1' THEN
		D_CHNSEL			<=	"00";												-- Dante channel mux select
		D_FIFO_WR			<=		'0';											-- Dante RX FIFO write ena
		CURRENT_STATE	<=		S0;											-- Async reset to RSYNC state
		TRANSFER				<=		'0';											-- Make sure no transfer occur
		ENA_XFER				<=		'0';											--
		STATE					<=	"000";											-- State output to zero
		TCNT					:=		0;												-- Set pulse count to zero	
	ELSIF RISING_EDGE(D_SCLK) THEN										--
		CURRENT_STATE	<=	NEXT_STATE;								--
		STATE					<=	STATE_XFER;								-- Output reflects state
		TRANSFER				<=	XFER_ENA;									--
			IF (TRANSFER	=	'1') THEN											--
				ENA_XFER		<=	NOT ENA_XFER;								-- Toggle
					IF(ENA_XFER	=	'1')THEN										-- During ENA_XFER HI
						TCNT 	:=	TCNT + 1;										-- Inc the channel counter
					ELSE																		--
						TCNT	:=	TCNT;											-- Otherwise just hold the count
					END IF;																	--
			ELSE																				-- 
				TCNT 		:=	0;														-- Else keep tcnt zero  
				ENA_XFER	<=	'0';													-- Keep FIFO write zero
			END IF;																			--
		PULSECOUNT		<= CONV_STD_LOGIC_VECTOR(TCNT,3);	-- Async combinatorial section uses this to shut off the transfers
		D_CHNSEL(0)	<=	PULSECOUNT(0);								-- Dante RX FIFO mux select 0
		D_CHNSEL(1)	<=	PULSECOUNT(1);								-- Dante RX FIFO mux select 1
		D_FIFO_WR		<=	ENA_XFER;										-- Dante RX FIFO write enable
	END IF;
END PROCESS REGISTERS;

--===================================================================================================
-- ASYNC PROCESS
--===================================================================================================
TRANSITIONS: PROCESS(CURRENT_STATE, PULSECOUNT, ENA_XFER, D_RESYNC, D_CHNCNT, D_CHNHLD)
BEGIN
	CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>						-- STATE 0
		XFER_ENA	<=		'0';
			IF ( D_RESYNC	=	'1' AND D_CHNCNT =	CONV_STD_LOGIC_VECTOR(14,4) ) THEN
				NEXT_STATE	<=		S1;
				STATE_XFER	<=	"001";
			ELSE
				NEXT_STATE	<=		S0;
				STATE_XFER	<=	"000";
			END IF;
--===================================================================================================
		WHEN S1	=>						-- STATE 1
		XFER_ENA	<=		'0';
			IF ( D_CHNCNT =	CONV_STD_LOGIC_VECTOR(15,4) ) THEN
				NEXT_STATE	<=		S2;
				STATE_XFER	<=	"010";
			ELSE
				NEXT_STATE	<=		S1;
				STATE_XFER	<=	"001";
			END IF;
--===================================================================================================
		WHEN S2	=>						-- STATE 2
		XFER_ENA	<=		'0';
			IF ( D_CHNHLD = '1' ) THEN
				NEXT_STATE	<=		S3;
				STATE_XFER	<=	"011";
			ELSE
				NEXT_STATE	<=		S2;
				STATE_XFER	<=	"010";
			END IF;
--===================================================================================================
		WHEN S3	=>						-- STATE 3
		XFER_ENA	<=		'0';
			IF ( D_CHNHLD = '0' ) THEN
				NEXT_STATE	<=		S4;
				STATE_XFER	<=	"100";
			ELSE
				NEXT_STATE	<=		S3;
				STATE_XFER	<=	"011";
			END IF;
--===================================================================================================
		WHEN S4	=>						-- STATE 4
		XFER_ENA	<=		'0';
			IF ( PULSECOUNT = "001" AND ENA_XFER = '1' ) THEN
					IF ( D_CHNCNT =	CONV_STD_LOGIC_VECTOR(14,4) ) THEN
						NEXT_STATE	<=		S5;
						STATE_XFER	<=	"101";
					ELSE
						NEXT_STATE	<=		S2;
						STATE_XFER	<=	"010";
					END IF;
			ELSE
				XFER_ENA		<=		'1';
				NEXT_STATE	<=		S4;
				STATE_XFER	<=	"100";
			END IF;
--===================================================================================================
		WHEN S5	=>						-- STATE 5
		XFER_ENA	<=		'0';
			IF ( D_RESYNC	=	'1' ) THEN
				IF( D_CHNCNT =	CONV_STD_LOGIC_VECTOR(14,4) ) THEN
					NEXT_STATE	<=		S2;
					STATE_XFER	<=	"010";
				ELSE
					NEXT_STATE	<=		S0;
					STATE_XFER	<=	"000";
				END IF;
			ELSE
				NEXT_STATE	<=		S5;
				STATE_XFER	<=	"101";
			END IF;
--===================================================================================================
--===================================================================================================
	END CASE;
END PROCESS TRANSITIONS;

END BEHAVE;
