--===================================================================================================
--	Description											:	Infineon address decoder for KRL Dante
--																:
--	Version													:	1.00
--	Date														:	Mar 2, 2010
--	Author													:	Anthony Medina
--	Company												:	Sierra Automated Systems
--																:
--	Address range for FPGA							:	
--	Address range for external CS thru FPGA	:	
--																:
--																:
--																:
--===================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.numeric_std.ALL;

ENTITY KRL_DANTE_ADDR_DECODE is
	PORT (
				CPU_ADDR	:IN	STD_LOGIC_VECTOR	(6 DOWNTO 0);
				CPU_DATA	:IN	STD_LOGIC_VECTOR	(7 DOWNTO 0);
				CPU_CSn		:IN	STD_LOGIC;
				CPU_RDn		:IN	STD_LOGIC;
				CPU_WRn		:IN	STD_LOGIC;
				FPGA_Clk		:IN	STD_LOGIC;
				FPGA_Rst		:IN	STD_LOGIC;
				DevAddr		:OUT	STD_LOGIC_VECTOR(6 DOWNTO 0);
				DevData		:OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
				DevRd			:OUT	STD_LOGIC;
				DevWr			:OUT	STD_LOGIC;
				DevRdSel		:OUT	STD_LOGIC_VECTOR(7 downto 0);
				DevWrSel		:OUT	STD_LOGIC_VECTOR(7 downto 0);
				RdOE			:OUT	STD_LOGIC;
				RdSel			:OUT	STD_LOGIC_VECTOR(2 downto 0)
		);
END ENTITY KRL_DANTE_ADDR_DECODE;

ARCHITECTURE BEHAVE OF KRL_DANTE_ADDR_DECODE IS

SIGNAL addr					:STD_LOGIC_VECTOR(6 DOWNTO 0);
SIGNAL RdOut				:STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL RdOutEn			:STD_LOGIC;
SIGNAL RdMuxSel			:STD_LOGIC_VECTOR(2 downto 0);
SIGNAL WrOut				:STD_LOGIC_VECTOR(7 downto 0);

BEGIN

PROCESS (FPGA_Clk, FPGA_Rst)
BEGIN
	IF FPGA_Rst = '1' THEN						--	Make sure everything is zero at startup
      DevRdSel	<= (others => '0');		--	Read device select
      DevWrSel	<= (others => '0');		--	Write device select
		DevAddr	<= (others => '0');		--	Device address for reading/writing
		DevData	<= (others => '0');		--	Device data input when writing
		DevRd		<= '0';							--	Device read, reclocked MPU Rd
		DevWr		<= '0';							--	Device write, reclocked MPU Wr
		RdOE		<= '1';							--	TriState BiDir IO
		RdSel		<= (others => '0');		--	Default select zero

	ELSIF rising_edge (FPGA_Clk) THEN	--	Start checking MPU address and decode it
		DevRdSel	<= RdOut;					--	Read device select
		DevWrSel	<= WrOut;					--	Write device select
		DevAddr	<= CPU_ADDR;				--	Re-Clk MPU Addr	
		DevData	<= CPU_DATA;				--	Re-Clk MPU Data
		DevRd		<= CPU_RDn;				--	Re-Clk MPU Rd
		DevWr		<= CPU_WRn;				--	Re-Clk MPU Wr
		RdSel		<= RdMuxSel;				--
		RdOE		<= RdOutEn;					--
	
	END IF;
END PROCESS;

PROCESS (CPU_CSn, CPU_ADDR, CPU_RDn, CPU_WRn, FPGA_Rst)
	CONSTANT BASE_RW_START_CS0	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#00#;	-- RD ADDR, START OF WR ADDR; CS0
	CONSTANT BASE_RW_START_CS1	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#10#;	-- RD ADDR, START OF WR ADDR; CS1
	CONSTANT BASE_RW_START_CS2	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#20#;	-- RD ADDR, START OF WR ADDR; CS2
	CONSTANT BASE_RW_START_CS3	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#30#;	-- RD ADDR, START OF WR ADDR; CS3
	CONSTANT BASE_RW_START_CS4	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#40#;	-- RD ADDR, START OF WR ADDR; CS4
	CONSTANT BASE_RW_START_CS5	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#50#;	-- RD ADDR, START OF WR ADDR; CS5
	CONSTANT BASE_RW_START_CS6	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#60#;	-- RD ADDR, START OF WR ADDR; CS6
	CONSTANT BASE_RW_START_CS7	:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#70#;	-- RD ADDR, START OF WR ADDR; CS7
	CONSTANT WR_ADDR_RANGE		:	INTEGER RANGE 16#7F# DOWNTO 0	:= 16#0F#;	-- WRITE ADDRESS INCREMENT RANGE
BEGIN
--###################################################################################################
--===================================================================================================
--	This process takes in the MPU Address and Chip Select to decode the proper device for reading/writing.
--	These are clocked in at the pins, then combinatorial logic decides which device to select.
--	Once the device is selected it is then re-clocked to eliminate any noise or glitches and presented to the
--	proper device.
--===================================================================================================
--###################################################################################################

	addr	<= CPU_ADDR;	-- Take in the MPU Address for decoding.
--===================================================================================================
--	Read Device select section:														DEFAULT VALUE AT STARTUP
--	DevRead(0):	FPGA Type
--	DevRead(1):	FPGA Version
--	DevRead(2):	Status 1
--	DevRead(3):	Status 2
--	DevRead(4):	Control Reg
--	DevRead(5):	Error Status
--	DevRead(6):	GE Status
--	DevRead(7):	Reserved
--===================================================================================================
IF(FPGA_Rst = '0') THEN
			--	Decode 0
	if((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS0,7) and CPU_CSn = '0' and CPU_RDn = '0')) then		--
			RdOut		<= conv_std_logic_vector(16#01#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(0,3);
			--	Decode 1
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS1,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#02#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(1,3);
			--	Decode 2
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS2,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#04#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(2,3);
			--	Decode 3
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS3,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#08#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(3,3);
			--	Decode 4
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS4,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#10#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(4,3);
			--	Decode 5
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS5,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#20#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(5,3);
			--	Decode 6
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS6,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#40#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(6,3);
			--	Decode 7
	elsif((addr = CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS7,7) and CPU_CSn = '0' and CPU_RDn = '0')) then	--
			RdOut		<= conv_std_logic_vector(16#80#,8);
			RdOutEn	<= '0';											-- Enable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(7,3);
	else
			RdOut		<= (others => '0');						-- All goes to zero
			RdOutEn	<= '1';											-- Disable out BiDir IO
			RdMuxSel	<= conv_std_logic_vector(0,3);
	END IF;
--===================================================================================================
--	Write Device select section:
--	DevWrite(0):	DANTE RX FIFO FRAME DELAY REGISTER
--	DevWrite(1):	DANTE TX FIFO FRAME DELAY REGISTER
--	DevWrite(2):	CONTROL REG
--	DevWrite(3):	SAMPLE RATE CHANGE TOGGLE
--	DevWrite(4):
--	DevWrite(5):
--	DevWrite(6):
--	DevWrite(7):
--===================================================================================================
			--	Decode 0
	IF( addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS0,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS0 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#01#,8);
			--	Decode 1
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS1,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS1 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#02#,8);
			--	Decode 2
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS2,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS2 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#04#,8);
			--	Decode 3
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS3,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS3 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#08#,8);
			--	Decode 4
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS4,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS4 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#10#,8);
			--	Decode 5
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS5,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS5 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#20#,8);
			--	Decode 6
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS6,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS6 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#40#,8);
			--	Decode 7
	elsif(addr >= CONV_STD_LOGIC_VECTOR(BASE_RW_START_CS7,7) and addr <= CONV_STD_LOGIC_VECTOR((BASE_RW_START_CS7 + WR_ADDR_RANGE),7) and CPU_CSn = '0' and CPU_WRn = '0') then	--
			WrOut	<= conv_std_logic_vector(16#80#,8);
	else
	      WrOut	<= (others => '0');					-- All goes to zero
	END IF;

ELSE
	RdOut		<= (others => '0');					-- All goes to zero
	RdOutEn	<= '1';										-- Disable out BiDir IO
	RdMuxSel	<= conv_std_logic_vector(0,3);	--
	WrOut		<= (others => '0');					-- All goes to zero
END IF;

END PROCESS;      
     
end architecture BEHAVE;
